@@ -2,7 +2,6 @@ require 'puppet/application'
 
 class Puppet::Application::Inspect < Puppet::Application
 
-  should_parse_config
   run_mode :agent
 
   option("--debug","-d")
@@ -18,7 +17,7 @@ class Puppet::Application::Inspect < Puppet::Application
   end
 
   def help
-    <<-HELP
+    <<-'HELP'
 
 puppet-inspect(8) -- Send an inspection report
 ========
@@ -118,6 +117,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
       end
 
       @report.configuration_version = catalog.version
+      @report.environment = Puppet[:environment]
 
       inspect_starttime = Time.now
       @report.add_times("config_retrieval", inspect_starttime - retrieval_starttime)
@@ -135,8 +135,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
         begin
           audited_resource = ral_resource.to_resource
         rescue StandardError => detail
-          puts detail.backtrace if Puppet[:trace]
-          ral_resource.err "Could not inspect #{ral_resource}; skipping: #{detail}"
+          ral_resource.log_exception(detail, "Could not inspect #{ral_resource}; skipping: #{detail}")
           audited_attributes.each do |name|
             event = ral_resource.event(
                                        :property => name,
@@ -164,7 +163,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
         end
         if Puppet[:archive_files] and ral_resource.type == :file and audited_attributes.include?(:content)
           path = ral_resource[:path]
-          if File.readable?(path)
+          if ::File.readable?(path)
             begin
               dipper.backup(path)
             rescue StandardError => detail
@@ -182,8 +181,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
       begin
         Puppet::Transaction::Report.indirection.save(@report)
       rescue => detail
-        puts detail.backtrace if Puppet[:trace]
-        Puppet.err "Could not send report: #{detail}"
+        Puppet.log_exception(detail, "Could not send report: #{detail}")
       end
     end
   end
