@@ -39,7 +39,10 @@ class Puppet::Node::Ldap < Puppet::Indirector::Ldap
     names.each do |name|
       next unless info = name2hash(name)
 
-      break if node = info2node(request.key, info)
+      merge_parent(info) if info[:parent]
+      info[:environment] ||= request.environment.to_s
+      node = info2node(request.key, info)
+      break
     end
 
     node
@@ -60,6 +63,8 @@ class Puppet::Node::Ldap < Puppet::Indirector::Ldap
     ldapsearch(filter) { |entry| infos << entry2hash(entry, request.options[:fqdn]) }
 
     return infos.collect do |info|
+      merge_parent(info) if info[:parent]
+      info[:environment] ||= request.environment.to_s
       info2node(info[:name], info)
     end
   end
@@ -195,8 +200,6 @@ class Puppet::Node::Ldap < Puppet::Indirector::Ldap
 
   # Take a name and a hash, and return a node instance.
   def info2node(name, info)
-    merge_parent(info) if info[:parent]
-
     node = Puppet::Node.new(name)
 
     add_to_node(node, info)
