@@ -2,16 +2,29 @@ require 'puppet/indirector/terminus'
 
 # The base class for YAML indirection termini.
 class Puppet::Indirector::Yaml < Puppet::Indirector::Terminus
+  if defined?(::Psych::SyntaxError)
+    YamlLoadExceptions = [::StandardError, ::ArgumentError, ::Psych::SyntaxError]
+  else
+    YamlLoadExceptions = [::StandardError, ::ArgumentError]
+  end
+
   # Read a given name's file in and convert it from YAML.
   def find(request)
     file = path(request.key)
     return nil unless FileTest.exist?(file)
 
+    yaml = nil
     begin
-      return from_yaml(File.read(file))
+      yaml = ::File.read(file)
     rescue => detail
       raise Puppet::Error, "Could not read YAML data for #{indirection.name} #{request.key}: #{detail}"
     end
+
+    begin
+      return from_yaml(yaml)
+    rescue *YamlLoadExceptions => detail
+      raise Puppet::Error, "Could not parse YAML data for #{indirection.name} #{request.key}: #{detail}"
+    end
   end
 
   # Convert our object to YAML and store it to the disk.
@@ -26,7 +39,9 @@ class Puppet::Indirector::Yaml < Puppet::Indirector::Terminus
     Dir.mkdir(basedir) unless FileTest.exist?(basedir)
 
     begin
-      File.open(file, "w", 0660) { |f| f.print to_yaml(request.instance) }
+      Puppet::Util.replace_file(file, 0660) do |f|
+        f.print to_yaml(request.instance)
+      end
     rescue TypeError => detail
       Puppet.err "Could not save #{self.name} #{request.key}: #{detail}"
     end
