@@ -1,5 +1,6 @@
 require 'puppet'
 require 'puppet/util/classgen'
+require 'puppet/node/environment'
 
 # Methods dealing with Type management.  This module gets included into the
 # Puppet::Type class, it's just split out here for clarity.
@@ -38,8 +39,8 @@ module Manager
     end
 
     # First make sure we don't have a method sitting around
-    name = symbolize(name)
-    newmethod = "new#{name.to_s}"
+    name = name.intern
+    newmethod = "new#{name}"
 
     # Used for method manipulation.
     selfobj = singleton_class
@@ -61,10 +62,9 @@ module Manager
 
     # Then create the class.
 
-          klass = genclass(
-        name,
+    klass = genclass(
+      name,
       :parent => (parent || Puppet::Type),
-        
       :overwrite => true,
       :hash => @types,
       :attributes => options,
@@ -87,14 +87,11 @@ module Manager
 
     # Now set up autoload any providers that might exist for this type.
 
-          klass.providerloader = Puppet::Util::Autoload.new(
-        klass,
-        
-      "puppet/provider/#{klass.name.to_s}"
-    )
+    klass.providerloader = Puppet::Util::Autoload.new(klass, "puppet/provider/#{klass.name.to_s}")
 
-    # We have to load everything so that we can figure out the default type.
+    # We have to load everything so that we can figure out the default provider.
     klass.providerloader.loadall
+    klass.providify unless klass.providers.empty?
 
     klass
   end
@@ -103,11 +100,7 @@ module Manager
   def rmtype(name)
     # Then create the class.
 
-          klass = rmclass(
-        name,
-        
-      :hash => @types
-    )
+    klass = rmclass(name, :hash => @types)
 
     singleton_class.send(:remove_method, "new#{name}") if respond_to?("new#{name}")
   end
@@ -116,28 +109,30 @@ module Manager
   def type(name)
     @types ||= {}
 
-    name = name.to_s.downcase.to_sym
+    # We are overwhelmingly symbols here, which usually match, so it is worth
+    # having this special-case to return quickly.  Like, 25K to 300 symbols to
+    # strings in this method. --daniel 2012-07-17
+    return @types[name] if @types[name]
 
-    if t = @types[name]
-      return t
-    else
-      if typeloader.load(name)
-        Puppet.warning "Loaded puppet/type/#{name} but no class was created" unless @types.include? name
-      end
-
-      return @types[name]
+    # Try mangling the name, if it is a string.
+    if name.is_a? String
+      name = name.downcase.intern
+      return @types[name] if @types[name]
     end
+
+    # Try loading the type.
+    if typeloader.load(name, Puppet::Node::Environment.current)
+      Puppet.warning "Loaded puppet/type/#{name} but no class was created" unless @types.include? name
+    end
+
+    # ...and I guess that is that, eh.
+    return @types[name]
   end
 
   # Create a loader for Puppet types.
   def typeloader
     unless defined?(@typeloader)
-
-            @typeloader = Puppet::Util::Autoload.new(
-        self,
-        
-        "puppet/type", :wrap => false
-      )
+      @typeloader = Puppet::Util::Autoload.new(self, "puppet/type", :wrap => false)
     end
 
     @typeloader
