@@ -29,11 +29,12 @@ in your manifest, including defined components.
     params.sort { |a,b|
       a.to_s <=> b.to_s
     }.each { |param|
-      str += paramwrap(param.to_s, scrub(Puppet::Type.metaparamdoc(param)), :level => 3)
+      str << markdown_header(param.to_s, 3)
+      str << scrub(Puppet::Type.metaparamdoc(param))
+      str << "\n\n"
     }
   rescue => detail
-    puts detail.backtrace
-    puts "incorrect metaparams: #{detail}"
+    Puppet.log_exception(detail, "incorrect metaparams: #{detail}")
     exit(1)
   end
 
